"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.RemoteMethodApi = void 0;
const coderitter_api_remote_method_call_1 = require("coderitter-api-remote-method-call");
const knight_log_1 = require("knight-log");
let log = new knight_log_1.Log('coderitter-api-remote-method-api/RemoteMethodApi.ts');
/**
 * A remote method call API. It is a simple mapping from a method name to
 * an endpoint which calls the method.
 */
class RemoteMethodApi {
    constructor() {
        this.methods = {};
    }
    /**
     * Get all the message id that can be handled
     */
    get methodNames() {
        return Object.keys(this.methods);
    }
    /**
     * Executes the method which was called remotely.
     *
     * @param remoteMethodCall May be on object of type RemoteMethodCall or a JSON string containing an object matching the type RemoteMethodCall
     */
    callMethod(remoteMethodCall) {
        return __awaiter(this, void 0, void 0, function* () {
            let l = log.mt('callMethod');
            l.lib('remoteMethodCall =', remoteMethodCall);
            let method = remoteMethodCall.method;
            if (method != undefined && method in this.methods) {
                let methodCall = this.methods[method];
                try {
                    if (typeof methodCall === 'function') {
                        l.lib('Remote method call handler is a function');
                        let result = yield methodCall(remoteMethodCall);
                        l.lib('result =', result);
                        return result;
                    }
                    else {
                        l.lib('Remote method call handler implements interface LocalMethodCall');
                        let result = yield methodCall.callMethod(remoteMethodCall);
                        l.lib('result =', result);
                        return result;
                    }
                }
                catch (e) {
                    l.error('There was an error executing the called remote method', e);
                    return this.onMethodError(e, remoteMethodCall);
                }
            }
            else {
                l.warn(`Remote method '${method}' not found.`, this.methodNames);
                return this.onRemoteMethodNotSupported(remoteMethodCall);
            }
        });
    }
    onMethodError(e, remoteMethodCall) {
        return coderitter_api_remote_method_call_1.Result.remoteError(`There was an error with your request. We just were informed that it happened and we will look into the issue. Please try again later.`);
    }
    onRemoteMethodNotSupported(remoteMethodCall) {
        return coderitter_api_remote_method_call_1.Result.remoteError(`Remote method '${remoteMethodCall.method}' not found.`);
    }
}
exports.RemoteMethodApi = RemoteMethodApi;
